'''

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
'''
import numpy
from os import path
from time import strftime
import matplotlib.pyplot as pylab

from dsc_suite.analyses.standards import COLORS
from dsc_suite.gui.config import DATA_STRUCTURE_LIST,BENCHMARK_LIST
from dsc_suite.tools.toolpaths import FOLDER_FOR_DATA, FOLDER_FOR_PLOTS

def plot_scatter_plot_operation(filenames, information, parameter, create_png=False):
    '''
    
    @param filenames: filenames relative to the default folder of data
    @type filenames: list
    @param information: information about the files in filenames extracted from XML data base
    @type information: dictionary
    @param parameter: parameters for the diagram layout
    @type parameter: dictionary
    @param create_png: create a png file(True) from diagram or show interactive mode(False)
    @type create_png: boolean
    '''
    if len(filenames) != 3:
        #raise Error
        return None
        pass
    
    #get different dictionaries
    labels = parameter["labels"]
    sizes = parameter["sizes"]
    axes = parameter["axes"]
    
    #get information about lables
    title = labels["title"]
    legend_position = labels["legend-position"]
    legend_title = labels["legend-title"]
    legend_parameter = labels["legend-parameter"]
    xlabel = labels["x-label"]
    ylabel = labels["y-label"]
    
    #get information about size
    width = sizes["dimensions"]["width"]
    length = sizes["dimensions"]["length"]
    left = sizes["margins"]["left"]
    right = sizes["margins"]["right"]
    top = sizes["margins"]["top"]
    bottom = sizes["margins"]["bottom"]
    
    #get information about axes
    if axes["x-axe"]["xmin"] == "auto":
        xmin = 0.0
    else:
        xmin = float(axes["x-axe"]["xmin"])
    if axes["x-axe"]["xmax"] == "auto":
        xsearch = True
        xmax = 0.0
    else:
        xmax = float(axes["x-axe"]["xmax"])
        xsearch = False
    if axes["y-axe"]["ymin"] == "auto":
        ymin = 0.0
    else:
        ymin = float(axes["y-axe"]["ymin"])
    if axes["y-axe"]["ymax"] == "auto":
        ymax = 0.0
        ysearch= True 
    else:
        ymax = float(axes["y-axe"]["ymax"])
        ysearch = False
  
    #create figure
    pylab.figure(1,(length,width))
    pylab.subplots_adjust(left,bottom,right,top)
    pylab.ticklabel_format(style = "sci", scilimits = (-2,5))
    
    print_dict = {}
    data_structure = None
    
    for file in filenames:
        dictionary = information[file]["cost_criteria"]
        benchmark = BENCHMARK_LIST[information[file]["benchmark"]][0](information[file]["benchmark"])
        benchmark.to_3d()
        data_structure = DATA_STRUCTURE_LIST[information[file]["datastructure"]][0](benchmark)
        for key in dictionary.keys():
            if str(dictionary[key]["filename"]) == str(file):
                print_dict.update({key : numpy.load(path.join(FOLDER_FOR_DATA, file))})
    
    if "ITER" not in print_dict.keys():
        return None
    
    criteria = print_dict.keys()
    criteria.remove("ITER")
    
    basics_x = []
    basics_y = []
    
    derived_x = []
    derived_y = []
        
    ops = data_structure.get_operations()
    keys = ops.keys()
    keys.sort()
    single_label, index = None, None
    if legend_parameter in keys:
        single_label = legend_parameter
        index = keys.index(legend_parameter)
        
    for i in range(0,len(keys),1):
        derived_x.append([])
        derived_y.append([])
    f = open("output.txt", "w")
    for key in print_dict.keys():
        print >> f,key
        for i in range(0,numpy.alen(print_dict[key]),1):
            print >> f, print_dict[key][i]
        print "-------"
    f.close()
    print criteria
    print numpy.alen(print_dict[criteria[1]])
    print numpy.alen(print_dict[criteria[0]])
    print numpy.alen(print_dict["ITER"])
    
    for i in range(0,len(print_dict["ITER"]),1):  
        if print_dict["ITER"][i].startswith("b-"):
            if print_dict[criteria[0]][i] > xmax and xsearch:
                xmax = print_dict[criteria[0]][i]*1.01
            basics_x.append(print_dict[criteria[0]][i])
            if print_dict[criteria[1]][i] > ymax and ysearch:
                ymax = print_dict[criteria[1]][i]*1.01
            basics_y.append(print_dict[criteria[1]][i])
            
        elif print_dict["ITER"][i].startswith("d-"):
            code = print_dict["ITER"][i].split("-")
            if int(code[4]) != 0:
                continue
            if print_dict[criteria[0]][i] > xmax and xsearch:
                xmax = print_dict[criteria[0]][i]*1.01
            derived_x[int(print_dict["ITER"][i].split("-")[1])].append(print_dict[criteria[0]][i])
            if print_dict[criteria[1]][i] > ymax and ysearch:
                ymax = print_dict[criteria[1]][i]*1.01
            derived_y[int(print_dict["ITER"][i].split("-")[1])].append(print_dict[criteria[1]][i])
    
    if single_label == None:
        for i in range(0,len(derived_x),1):
            pylab.plot(derived_x[i],derived_y[i],",",color=COLORS[i],label=keys[i])
    else:
        pylab.plot(derived_x[index],derived_y[index],",",color=COLORS[index],label=keys[index])
    pylab.plot(basics_x ,basics_y,"o", color="yellow", label = "basic")
        
    
    #set axis names    
    pylab.xlabel(xlabel)
    pylab.ylabel(ylabel)
    #set axis limits
    pylab.axis([xmin, xmax, ymin, ymax])
    #set grid
    pylab.grid(True)
    #set title
    pylab.suptitle(title)
    #create legend --> color map
    pylab.legend(title = legend_title, loc = legend_position)
    #create png
    if create_png:
        i = 1
        filename = 'Scatter plot operation%s %02i.png' % (strftime("%Y-%m-%d %H%M%S"), i)
        while path.exists(FOLDER_FOR_PLOTS + '/' + filename):
            i += 1
            filename = 'Scatter plot operation %s %02i.png' % (strftime("%Y-%m-%d %H%M%S"), i)
        pylab.savefig(FOLDER_FOR_PLOTS + '/' + filename)
        pylab.close()
        return filename
    else:
        pylab.show()